/*
    ldapdiff
    Copyright (C) 2000-2008 Thomas.Reith@rhoen.de

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <lber.h>
#include <ldap.h>

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <syslog.h>
#include <ctype.h>

#include "ldapdiff.h"

void ldifopen(LDAP **pld)
{
 int            rc;
 int            ldapoption;
 char          *ldapuri     = NULL;
 LDAP          *ld;
 LDAPURLDesc    url;
 struct berval  passwd;
 struct berval *spasswd;

#ifndef USE_DEPRECATED_API
 memset(&url,0,sizeof(url));
 url.lud_scheme = "ldap";
 url.lud_host = ldifgetgconf(CONFLDAPHOST);
 url.lud_port = atoi(ldifgetgconf(CONFLDAPPORT));
 url.lud_scope = LDAP_SCOPE_DEFAULT;
 ldapuri = ldap_url_desc2str(&url);
 if((rc = ldap_initialize(&ld,ldapuri)) != LDAP_SUCCESS){
  ldiflog(LOG0,"ldap_initialize() failed: file: %s, line: %d",__FILE__,__LINE__);
  ldiflog(LOG0,"ldap_err2string(): %s",ldap_err2string(rc));
  exit(EXITLDERROR);
 }
#else
 if((ld = ldap_open(ldifgetgconf(CONFLDAPHOST),atoi(ldifgetgconf(CONFLDAPPORT)))) == NULL){
  ldiflog(LOG0,"ldap_open() failed: file: %s, line: %d",__FILE__,__LINE__);
  perror("perror(): ");
  exit(EXITLDERROR);
 }
#endif 

 ldapoption = LDAP_NO_LIMIT;
 if(ldap_set_option(ld,LDAP_OPT_SIZELIMIT,&ldapoption) == -1){
  ldiflog(LOG0,"ldap_set_option() failed: file: %s, line: %d",__FILE__,__LINE__);
  exit(EXITLDERROR);
 }

 ldapoption = LDAP_VERSION3;
 if(ldap_set_option(ld,LDAP_OPT_PROTOCOL_VERSION,&ldapoption) == -1){
  ldiflog(LOG0,"ldap_set_option() failed: file: %s, line: %d",__FILE__,__LINE__);
  exit(EXITLDERROR);
 }

 if(strcmp(ldifgetgconf(CONFMANAGEDSAIT),"yes") == 0){
  LDAPControl c;
  LDAPControl *ctrls[2];
  ctrls[0] = &c;
  ctrls[1] = NULL;

  c.ldctl_oid = LDAP_CONTROL_MANAGEDSAIT;
  c.ldctl_value.bv_val = NULL;
  c.ldctl_value.bv_len = 0;
  c.ldctl_iscritical = 0;

  rc = ldap_set_option( ld, LDAP_OPT_SERVER_CONTROLS, ctrls );

  if(rc != LDAP_OPT_SUCCESS){
   ldiflog(LOG0,"ldap_set_option() failed: file: %s, line: %d",__FILE__,__LINE__);
   exit(EXITLDERROR);
  }
 }

#ifndef USE_DEPRECATED_API
 passwd.bv_val = ldifgetgconf(CONFROOTPW);
 passwd.bv_len = strlen(passwd.bv_val);
 if((rc = ldap_sasl_bind_s(ld,ldifgetgconf(CONFROOTDN),NULL,&passwd,NULL,NULL,&spasswd)) != LDAP_SUCCESS){
  ldiflog(LOG0,"ldap_sasl_bind_s() failed: file: %s, line: %d",__FILE__,__LINE__);
#else
 if((rc = ldap_bind_s(ld,ldifgetgconf(CONFROOTDN),ldifgetgconf(CONFROOTPW),LDAP_AUTH_SIMPLE)) != LDAP_SUCCESS){
  ldiflog(LOG0,"ldap_bind_s() failed: file: %s, line: %d",__FILE__,__LINE__);
#endif
  ldiflog(LOG0,"ldap_err2string(): %s",ldap_err2string(rc));
  exit(EXITLDERROR);
 }

 *pld = ld;
}

void ldifclose(LDAP *ld)
{
 int rc;

#ifndef USE_DEPRECATED_API
 if((rc = ldap_set_option(ld,LDAP_OPT_SERVER_CONTROLS,NULL)) != LDAP_OPT_SUCCESS){
  ldiflog(LOG0,"ldap_set_option() failed: file: %s, line: %d",__FILE__,__LINE__);
  ldiflog(LOG0,"ldap_err2string(): %s",ldap_err2string(rc));
  exit(EXITLDERROR);
 }
 if((rc = ldap_unbind_ext_s(ld,NULL,NULL)) != LDAP_OPT_SUCCESS){
  ldiflog(LOG0,"ldap_unbind_ext() failed: file: %s, line: %d",__FILE__,__LINE__);
  ldiflog(LOG0,"ldap_err2string(): %s",ldap_err2string(rc));
  exit(EXITLDERROR);
 }
#else
 if(ldap_unbind(ld) == -1){
  ldiflog(LOG0,"ldap_unbind() failed: file: %s, line: %d",__FILE__,__LINE__);
  ldiflog(LOG0,"ldap_err2string(): %s",ldap_err2string(rc));
  exit(EXITLDERROR);
 }
#endif 
}
